# argv[1] = peaks_bed_file, argv[2] = scaffold_sizes_file, argv[3] = genome_sequence_file, argv[4] = output_file
import sys
import math
import numpy as np
from sklearn.neighbors import KernelDensity

def retrieve_lengths(peaks_bed_file):
    #retrieve fragment lengths from peak file
    peak_lengths = []
    for line in open(peaks_bed_file,'r'):
        scaffold, start, stop, pile, evalue = line.split('\t')
        length = int(stop) - int(start)
        peak_lengths.append(length)
    return peak_lengths

def estimate_scaffold_distribution(scaffold_sizes_file):
    #retrieve scaffold sizes, probability of drawing from scaffold = % of genome in scaffold
    scaffolds, scaffold_probabilities, sizes, total = [], [], [], 0.0
    for line in (open(scaffold_sizes_file,'r')):
        scaffold, length = line.split('\t')
        total = total + float(length)
        sizes.append([scaffold, float(length)])
    for scaffold in sizes:
        scaffold_probabilities.append(scaffold[1]/total)
        scaffolds.append(scaffold[0])
    cumulative = 0
    for prob in scaffold_probabilities:
        cumulative = cumulative + prob
    return scaffolds, scaffold_probabilities

def estimate_fragment_distribution(peak_lengths):
    #retrieve fragment lengths, use KDE to estimate fragment length probability
    length_array = np.asarray(peak_lengths)
    print 'Peaks mean: ', np.mean(length_array)
    print 'Peaks median: ', np.median(length_array)
    print 'Peaks std-dev: ', np.std(length_array)
    print 'Peaks 25th percentile: ', np.percentile(length_array, 25)
    print 'Peaks 75th percentile: ', np.percentile(length_array, 75)
    peak_distribution = KernelDensity(kernel='gaussian', bandwidth=10).fit(length_array)
    samples = peak_distribution.sample(1)
    return samples[0][0:10000]

def load_genome(genome):
    print 'Loading Genome.'
    ID, temp_seq, genome_dict, IDs = '', [], {}, []
    for line in genome:
        if '>' in line:
            ID = line[1:-1]
            IDs.append(ID)
            temp_seq.append([])
        else:
            temp_seq[-1].append(line.strip())
    for i, seq in enumerate(temp_seq):
        fasta = ''.join(temp_seq[-1])
        genome_dict[IDs[i]] = fasta
    return genome_dict

def check_random_fragments(frags):
    lengths = []
    for frag in frags:
        lengths.append(len(frag))
    length_array = np.asarray(lengths)
    print 'Frags mean: ', np.mean(length_array)
    print 'Frags median: ', np.median(length_array)
    print 'Frags std-dev: ', np.std(length_array)
    print 'Frags 25th percentile: ', np.percentile(length_array, 25)
    print 'Frags 75th percentile: ', np.percentile(length_array, 75)

def create_random_fragments(genome, scaffolds, scaffold_probabilities, fragments, out):
    genome_dict = load_genome(genome)
    frags = []
    for i, fragment in enumerate(fragments):
        scaffold = np.random.choice(scaffolds, replace=True, p=scaffold_probabilities)
        seq = genome_dict[scaffold]
        maximum = len(seq)-1000
        start = np.random.randint(0,maximum)
        random_frag = seq[int(start):int(start + fragment)]
        header = '>' + str(i) + '\n'
        out.write(header)
        out.write(random_frag)
        out.write('\n')
        frags.append(random_frag)
    check_random_fragments(frags)

out = open(sys.argv[4],'w')
genome = open(sys.argv[3], 'r')
scaffolds, scaffold_probabilities = estimate_scaffold_distribution(sys.argv[2])
peak_lengths = retrieve_lengths(sys.argv[1])
fragments = estimate_fragment_distribution(peak_lengths)
create_random_fragments(genome, scaffolds, scaffold_probabilities, fragments, out)
